<?php
/**
 * 2007-2020 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş. <devops@paytr.com>
 * @copyright PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 * International Registered Trademark & Property of PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 */

require_once(dirname(__FILE__) . '/PaytrTransaction.php');
require_once(dirname(__FILE__) . '/PaytrTransactionForEFTByNotification.php');

class PaytrNotification
{
    public $context;
    public $module;

    public function callback($params, $uniq_separator, $api_type)
    {
        // Check Hash
        $this->checkHash($params, $api_type);

        // Get ID Cart
        $id_cart = explode($uniq_separator, $params['merchant_oid'], 2);

        // Determine transaction class based on API type
        $transactionClass = $api_type === 'eft' ? 'PaytrTransactionForEFTByNotification' : 'PaytrTransaction';
        $transaction = $transactionClass::getTransactionByIdOidCartByOid($id_cart[1], $params['merchant_oid']);

        if (!$transaction) {
            die($api_type === 'eft' ? 'OK.' : 'OK');
        }

        // If transaction is already completed
        if ($transaction['is_complete'] && ($transaction['status'] == 'success' || $transaction['status'] == 'failed')) {
            echo 'OK';
            exit;
        }

        // Get Configurations
        $configurations = $this->getConfigurations($api_type);
        $conf_pending = $configurations['pending'];
        $conf_canceled = $configurations['canceled'];
        $conf_completed = $configurations['completed'];
        $conf_invoice = $api_type === 'iframe' ? $configurations['invoice'] : null;

        // Amounts
        $total_amount = round($params['total_amount'] / 100, 2);
        $payment_amount = round($params['payment_amount'] / 100, 2);

        $val = [
            'status' => $params['status'],
            'total_amount' => $total_amount,
            'payment_amount' => $payment_amount,
            'conf_pending' => $conf_pending,
            'conf_completed' => $conf_completed,
            'conf_canceled' => $conf_canceled,
            'merchant_oid' => $params['merchant_oid'],
            'payment_type' => $params['payment_type']
        ];

        if (array_key_exists('installment_count', $params)) {
            $val['installment_count'] = $params['installment_count'];
        }

        if ($params['status'] == 'failed') {
            $val['failed_reason_code'] = $params['failed_reason_code'];
            $val['failed_reason_msg'] = $params['failed_reason_msg'];
        }

        if ($conf_invoice !== null) {
            $val['conf_invoice'] = $conf_invoice;
        }

        // Check order if already placed before.
        $ps_order_id = $this->checkOrder((int)$id_cart[1]);

        if ($ps_order_id) {
            $this->handleExistingOrder($ps_order_id, $val, $transaction, $conf_pending, $transactionClass, $api_type);
        } elseif ($transaction['is_order'] && !$transaction['is_complete']) {
            $this->handleIncompleteOrder($id_cart[1], $val, $transaction, $conf_pending, $transactionClass, $api_type);
        } elseif (!$transaction['is_order']) {
            $this->handleNewOrder($val, $transaction, $conf_pending, $transactionClass, $api_type);
        }
    }

    /**
     * Get configurations based on API type
     *
     * @param string $api_type Either 'iframe' or 'eft'
     * @return array
     */
    private function getConfigurations($api_type)
    {
        $prefix = $api_type === 'eft' ? 'PAYTR_EFT_' : 'PAYTR_';
        return [
            'pending' => Configuration::get($prefix . 'PENDING'),
            'canceled' => Configuration::get($prefix . 'CANCELED'),
            'completed' => Configuration::get($prefix . 'COMPLETED'),
            'invoice' => $api_type === 'iframe' ? Configuration::get($prefix . 'INVOICE') : null
        ];
    }

    /**
     * Handle an existing order
     *
     * @param int $ps_order_id Existing order ID
     * @param array $val Values for updating
     * @param array $transaction Transaction data
     * @param int $conf_pending Pending order state ID
     * @param string $transactionClass Transaction class name
     * @param string $api_type API type
     */
    private function handleExistingOrder($ps_order_id, array $val, array $transaction, $conf_pending, $transactionClass, $api_type)
    {
        $order = new Order($ps_order_id);
        if ($order->current_state != $conf_pending) {
            $this->updateOrderState($order, $val, $transaction, $transactionClass);
            echo 'OK';
            exit;
        }

        $this->updateStatus($val, $order, $transaction['id_oid_cart'], 0, $api_type);
    }

    /**
     * Handle an incomplete order
     *
     * @param int $id_cart Cart ID
     * @param array $val Values for updating
     * @param array $transaction Transaction data
     * @param int $conf_pending Pending order state ID
     * @param string $transactionClass Transaction class name
     * @param string $api_type API type
     */
    private function handleIncompleteOrder($id_cart, array $val, array $transaction, $conf_pending, $transactionClass, $api_type)
    {
        $order_id = $this->checkOrder((int)$id_cart);
        if (!$order_id) {
            die('OK');
        }

        $order = new Order($order_id);
        if ($order->current_state != $conf_pending) {
            $this->updateOrderState($order, $val, $transaction, $transactionClass);
            echo 'OK';
            exit;
        }

        $this->updateStatus($val, $order, $id_cart, 0, $api_type);
    }

    /**
     * Handle a new order creation
     *
     * @param array $val Values for updating
     * @param array $transaction Transaction data
     * @param int $conf_pending Pending order state ID
     * @param string $transactionClass Transaction class name
     * @param string $api_type API type
     */
    private function handleNewOrder(array $val, array $transaction, $conf_pending, $transactionClass, $api_type)
    {
        if ($val['status'] == 'failed' && $val['failed_reason_code'] == 6) {
            echo 'OK';
            exit;
        }

        $cart = new Cart($transaction['id_oid_cart']);
        $currency = new Currency($cart->id_currency);
        $total = (float)$cart->getOrderTotal(true, Cart::BOTH);
        $customer = new Customer($cart->id_customer);

        $this->module->validateOrder($cart->id, $conf_pending, $total, $this->module->displayName, null, null, (int)$currency->id, false, $customer->secure_key);

        $order = new Order($this->module->currentOrder);

        $this->updateStatus($val, $order, $transaction['id_oid_cart'], 1, $api_type);
    }

    /**
     * Update order state if it's not pending
     *
     * @param Order $order Order object
     * @param array $val Values from callback
     * @param array $transaction Transaction data
     * @param string $transactionClass Transaction class name
     */
    private function updateOrderState(Order $order, array $val, array $transaction, $transactionClass)
    {
        if ($val['status'] == 'success' && $order->current_state == 12) { // State 12 = Out of Stock (Pre-Order Not Paid)
            $order->setCurrentState(9); // Out of Stock (Pre-Order Paid)
            $status = 'completed';
        } else {
            $status = 'done';
        }

        $transactionClass::updateTransactionIsComplete($transaction['id_oid_cart'], $val['merchant_oid'], 'success', $status, $val['total_amount'], 0);
    }

    public function iframeCallback($params, $uniq_separator)
    {
        $this->callback($params, $uniq_separator, 'iframe');
    }

    public function eftCallback($params, $uniq_separator)
    {
        $this->callback($params, $uniq_separator, 'eft');
    }

    protected function checkOrder($id_cart): int
    {
        return version_compare(_PS_VERSION_, '1.7.1.0', '>=')
            ? Order::getIdByCartId((int)$id_cart)
            : Order::getOrderByCartId((int)$id_cart);
    }

    protected function checkHash($params, $api_name): bool
    {
        $merchant = $this->getMerchantDetails($api_name);

        $hash_string = $params['merchant_oid'] . $merchant['salt'] . $params['status'] . $params['total_amount'];
        $created_hash = base64_encode(hash_hmac('sha256', $hash_string, $merchant['key'], true));

        if ($created_hash !== $params['hash']) {
            die('PAYTR notification failed: bad hash.');
        }

        return true;
    }

    /**
     * Get merchant details based on the API name
     *
     * @param string $api_name The API type ('iframe' or 'eft')
     * @return array Merchant details
     */
    private function getMerchantDetails($api_name): array
    {
        $prefix = $api_name === 'eft' ? 'PAYTR_EFT_' : 'PAYTR_';
        return [
            'id' => Configuration::get($prefix . 'MERCHANT_ID'),
            'key' => Configuration::get($prefix . 'MERCHANT_KEY'),
            'salt' => Configuration::get($prefix . 'MERCHANT_SALT'),
        ];
    }

    protected function updateStatus(array $val, Order $order, $id_cart, $is_notify, $api_name): void
    {
        if ($val['status'] === 'success') {
            $this->handleSuccessStatus($val, $order, $id_cart, $is_notify, $api_name);
        } elseif ($val['status'] === 'failed' && $val['failed_reason_code'] !== '6') {
            $this->handleFailedStatus($val, $order, $id_cart, $api_name);
        }

        echo 'OK';
        exit;
    }

    /**
     * Handle successful payment status update
     *
     * @param array $val Callback data
     * @param Order $order Order object
     * @param int $id_cart Cart ID
     * @param int $is_notify Notification flag
     * @param string $api_name API type ('iframe' or 'eft')
     */
    private function handleSuccessStatus(array $val, Order $order, $id_cart, $is_notify, $api_name): void
    {
        $this->updateOrderStateForSuccess($order, $val);
        $this->updateInvoiceIfNeeded($val, $order, $api_name);
        $this->updateTransactionForSuccess($id_cart, $val, $is_notify, $api_name);
    }

    /**
     * Update order state for successful payment
     *
     * @param Order $order Order object
     * @param array $val Callback data
     */
    private function updateOrderStateForSuccess(Order $order, array $val): void
    {
        if ($order->current_state == $val['conf_pending']) {
            $order->setCurrentState($val['conf_completed']);
        } elseif ($order->current_state == 12) { // State 12 = Out of Stock (Pre-Order Not Paid)
            $order->setCurrentState(9); // Out of Stock (Pre-Order Paid)
        }
    }

    /**
     * Update invoice with installment details if required
     *
     * @param array $val Callback data
     * @param Order $order Order object
     * @param string $api_name API type ('iframe' or 'eft')
     */
    private function updateInvoiceIfNeeded(array $val, Order $order, $api_name): void
    {
        if ($api_name !== 'eft' && isset($val['conf_invoice']) && $val['conf_invoice']) {
            $installment_amount = $val['total_amount'] - $val['payment_amount'];
            if ($installment_amount > 0) {
                $invoice = new OrderInvoice($order->invoice_number);
                if ($invoice) {
                    $this->addInstallmentNoteToInvoice($invoice, $order->id_currency, $installment_amount, $val['total_amount'], $val['installment_count'] ?? null);
                }
            }
        }
    }

    /**
     * Add installment details note to invoice
     *
     * @param OrderInvoice $invoice Invoice object
     * @param int $currency_id Currency ID
     * @param float $installment_amount Installment amount
     * @param float $total_amount Total amount
     * @param int|null $installment_count Installment count
     */
    private function addInstallmentNoteToInvoice(OrderInvoice $invoice, $currency_id, $installment_amount, $total_amount, $installment_count)
    {
        $currency = new Currency($currency_id);
        $note = '';

        if (version_compare(_PS_VERSION_, '1.7.6.0', '>=')) {
            $note .= "Vade Farkı : " . $this->context->currentLocale->formatPrice($installment_amount, $currency->iso_code) . "\n";
            $note .= "Vade Farkı Dahil Toplam Tutar : " . $this->context->currentLocale->formatPrice($total_amount, $currency->iso_code);
        } else {
            $note .= "Vade Farkı : " . Tools::displayPrice($installment_amount, $currency) . "\n";
            $note .= "Vade Farkı Dahil Toplam Tutar : " . Tools::displayPrice($total_amount, $currency);
        }

        if ($installment_count !== null) {
            $note .= "\nTaksit Sayısı : " . ($installment_count == 1 ? "Tek Çekim" : $installment_count);
        }

        PaytrTransaction::updateOrderInvoice($invoice->number, $invoice->id_order, $note);
    }

    /**
     * Update transaction for successful payment
     *
     * @param int $id_cart Cart ID
     * @param array $val Callback data
     * @param int $is_notify Notification flag
     * @param string $api_name API type ('iframe' or 'eft')
     */
    private function updateTransactionForSuccess($id_cart, array $val, $is_notify, $api_name): void
    {
        $transactionClass = $api_name === 'eft' ? 'PaytrTransactionForEFTByNotification' : 'PaytrTransaction';
        $transactionClass::updateTransactionIsComplete($id_cart, $val['merchant_oid'], $val['status'], 'completed', $val['total_amount'], $is_notify);
    }

    /**
     * Handle failed payment status update
     *
     * @param array $val Callback data
     * @param Order $order Order object
     * @param int $id_cart Cart ID
     * @param string $api_name API type ('iframe' or 'eft')
     */
    private function handleFailedStatus(array $val, Order $order, $id_cart, $api_name): void
    {
        $order->setCurrentState($val['conf_canceled']);
        $this->updateTransactionForFailure($id_cart, $val, $api_name);
    }

    /**
     * Update transaction for failed payment
     *
     * @param int $id_cart Cart ID
     * @param array $val Callback data
     * @param string $api_name API type ('iframe' or 'eft')
     */
    private function updateTransactionForFailure($id_cart, array $val, $api_name): void
    {
        $status_message = $val['failed_reason_code'] . " - " . $val['failed_reason_msg'];
        $transactionClass = $api_name === 'eft' ? 'PaytrTransactionForEFTByNotification' : 'PaytrTransaction';
        $transactionClass::updateTransactionIsComplete($id_cart, $val['merchant_oid'], $val['status'], $status_message, 0);
    }
}
