<?php
/**
 * 2007-2020 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş. <devops@paytr.com>
 * @copyright PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 * International Registered Trademark & Property of PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 */

require_once(dirname(__FILE__) . '/../../classes/PaytrTransaction.php');

class PaytrCheckoutPaymentModuleFrontController extends ModuleFrontController
{
    protected array $category_full = [];
    protected array $category_installment = [];

    public function initContent()
    {
        $this->display_column_left = false;
        $this->display_column_right = false;

        parent::initContent();

        $errors = null;
        $iframe_token = null;
        $paytr_params = [];
        $user_basket = $this->prepareUserBasket();

        $paytr_params = $this->preparePaytrParams($user_basket);
        $post_data = $this->generatePostData($paytr_params);

        if (function_exists('curl_version')) {
            $result = $this->executeCurlRequest($post_data);
            list($errors, $iframe_token) = $this->processCurlResponse($result, $paytr_params);
        } else {
            $errors = 'CURL Problem!';
        }

        $this->assignSmartyVariables($errors, $iframe_token);
        $this->setTemplateByVersion();
    }

    /**
     * Prepare user basket based on cart products
     *
     * @return array
     */
    private function prepareUserBasket(): array
    {
        $user_basket = [];
        if (Configuration::get('PAYTR_INSTALLMENT') != 13) {
            foreach ($this->context->cart->getProducts() as $product) {
                $user_basket[] = [$product['name'], $product['total_wt'], $product['cart_quantity']];
            }
        } else {
            $user_basket = $this->prepareCategoryBasedBasket();
        }
        return $user_basket;
    }

    /**
     * Prepare PayTR parameters for payment
     *
     * @param array $user_basket User's basket information
     * @return array
     */
    private function preparePaytrParams(array $user_basket): array
    {
        $installment_config = Configuration::get('PAYTR_INSTALLMENT');
        $max_installment = $installment_config == 13 ? $this->calculateCategoryBasedInstallment($user_basket) : (in_array($installment_config, range(0, 12)) ? $installment_config : 0);

        $customer = new Customer($this->context->cart->id_customer);
        $address = new Address($this->context->cart->id_address_invoice);
        $currency = new Currency($this->context->cart->id_currency);

        $paytr_params = [
            'max_installment' => $max_installment,
            'no_installment' => $max_installment == 1 ? 1 : 0,
            'merchant_id' => trim(Configuration::get('PAYTR_MERCHANT_ID')),
            'merchant_key' => trim(Configuration::get('PAYTR_MERCHANT_KEY')),
            'merchant_salt' => trim(Configuration::get('PAYTR_MERCHANT_SALT')),
            'user_ip' => $this->getIp(),
            'oid' => uniqid() . 'PAYTRYPS' . $this->context->cart->id,
            'email' => $customer->email,
            'payment_amount' => round($this->context->cart->getOrderTotal(true, Cart::BOTH) * 100),
            'user_basket' => base64_encode(json_encode($user_basket)),
            'user_name' => $customer->firstname . ' ' . $customer->lastname,
            'user_address' => $address->address1 . ' ' . $address->address2 . ' ' . $address->postcode . ' ' . $address->city . ' ' . $address->country,
            'user_phone' => $address->phone_mobile ? $address->phone_mobile : $address->phone,
            'currency' => Tools::strtoupper($currency->iso_code),
        ];

        return $paytr_params + $this->createHashAndToken($paytr_params);
    }

    /**
     * Prepare basket for category-based installment options
     *
     * @return array
     */
    private function prepareCategoryBasedBasket(): array
    {
        $user_basket = [];
        $this->loadCategoryInstallments();
        foreach ($this->context->cart->getProducts() as $product) {
            $user_basket[] = [$product['name'], $product['total_wt'], $product['cart_quantity']];
            $this->addProductCategoriesToInstallment($product['id_product']);
        }
        return $user_basket;
    }

    /**
     * Load category-based installment configurations
     */
    private function loadCategoryInstallments(): void
    {
        $conf_ci = Configuration::get('PAYTR_CI');
        if ($conf_ci) {
            $this->category_installment = unserialize($conf_ci);
        }
    }

    /**
     * Add product categories to installment calculation
     *
     * @param int $id_product Product ID
     */
    private function addProductCategoriesToInstallment(int $id_product): void
    {
        $query = Db::getInstance()->ExecuteS("SELECT id_category as 'category_id' FROM `" . _DB_PREFIX_ . "category_product` WHERE id_product = '" . $id_product . "'");
        foreach ($query as $item) {
            $this->category_full[$item['category_id']] = $this->catSearch($item['category_id']);
        }
    }

    /**
     * Calculate maximum installment based on category
     *
     * @param array $user_basket User's basket information
     * @return int
     */
    private function calculateCategoryBasedInstallment(array $user_basket): int
    {
        $installments = array_filter($this->category_full);
        return count($installments) > 0 ? min($installments) : 0;
    }

    /**
     * Create hash and PayTR token
     *
     * @param array $paytr_params PayTR parameters
     * @return array
     */
    private function createHashAndToken(array $paytr_params): array
    {
        $hash_str = implode('', [
            $paytr_params['merchant_id'],
            $paytr_params['user_ip'],
            $paytr_params['oid'],
            $paytr_params['email'],
            $paytr_params['payment_amount'],
            $paytr_params['user_basket'],
            $paytr_params['no_installment'],
            $paytr_params['max_installment'],
            $paytr_params['currency']
        ]);
        $paytr_token = base64_encode(hash_hmac('sha256', $hash_str . $paytr_params['merchant_salt'], $paytr_params['merchant_key'], true));
        $order_token = $this->generateOrderToken($paytr_params['merchant_id']);

        return [
            'paytr_token' => $paytr_token,
            'order_token' => $order_token
        ];
    }

    /**
     * Generate an order token
     *
     * @param string $merchant_id Merchant ID
     * @return string
     */
    private function generateOrderToken(string $merchant_id): string
    {
        return base64_encode(hash_hmac('sha256', md5(uniqid(mt_rand(), true)), $merchant_id, true));
    }

    /**
     * Generate post data for PayTR API request
     *
     * @param array $paytr_params PayTR parameters
     * @return array
     */
    private function generatePostData(array $paytr_params): array
    {
        $lang_arr = ['tr', 'tr-tr', 'tr_tr', 'turkish', 'turk', 'türkçe', 'turkce', 'try', 'tl'];
        return [
            'merchant_id' => $paytr_params['merchant_id'],
            'user_ip' => $paytr_params['user_ip'],
            'merchant_oid' => $paytr_params['oid'],
            'email' => $paytr_params['email'],
            'payment_amount' => $paytr_params['payment_amount'],
            'paytr_token' => $paytr_params['paytr_token'],
            'user_basket' => $paytr_params['user_basket'],
            'debug_on' => 1,
            'no_installment' => $paytr_params['no_installment'],
            'max_installment' => $paytr_params['max_installment'],
            'user_name' => $paytr_params['user_name'],
            'user_address' => $paytr_params['user_address'],
            'user_phone' => $paytr_params['user_phone'],
            'lang' => in_array(Tools::strtolower($this->context->language->iso_code), $lang_arr) ? 'tr' : 'en',
            'currency' => $paytr_params['currency'],
            'merchant_ok_url' => $this->context->link->getModuleLink('paytrcheckout', 'confirmation', ['otoken' => urlencode($paytr_params['order_token'])]),
            'merchant_fail_url' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'index.php?controller=order'
        ];
    }

    /**
     * Execute CURL request to PayTR API
     *
     * @param array $post_data Data to post
     * @return string|false
     */
    private function executeCurlRequest(array $post_data)
    {
        $ch = curl_init();
        curl_setopt_array($ch, [
            CURLOPT_URL => "https://www.paytr.com/odeme/api/get-token",
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_POST => 1,
            CURLOPT_POSTFIELDS => $post_data,
            CURLOPT_FRESH_CONNECT => true,
            CURLOPT_TIMEOUT => 90,
            CURLOPT_CONNECTTIMEOUT => 90,
            CURLOPT_SSLVERSION => 6,
        ]);
        $result = @curl_exec($ch);
        if ($result === false) {
            $errors = "PAYTR IFRAME connection error. err: " . curl_error($ch);
        }
        curl_close($ch);
        return $result;
    }

    /**
     * Process CURL response from PayTR API
     *
     * @param string|false $result CURL execution result
     * @param array $paytr_params PayTR parameters for transaction creation
     * @return array
     */
    private function processCurlResponse($result, array $paytr_params): array
    {
        $errors = null;
        $iframe_token = null;
        if ($result === false) {
            $errors = "PAYTR IFRAME connection error.";
        } else {
            $response = json_decode($result, true);
            if ($response['status'] == 'success') {
                $iframe_token = $response['token'];
                $this->saveTransaction($paytr_params);
            } else {
                $errors = "PAYTR IFRAME failed. reason: " . $response['reason'];
            }
        }
        return [$errors, $iframe_token];
    }

    /**
     * Save PayTR transaction details
     *
     * @param array $paytr_params PayTR parameters
     */
    private function saveTransaction(array $paytr_params): void
    {
        $paytr_transaction = new PaytrTransaction();
        $paytr_transaction->id_oid_cart = $this->context->cart->id;
        $paytr_transaction->order_token = $paytr_params['order_token'];
        $paytr_transaction->merchant_oid = $paytr_params['oid'];
        $paytr_transaction->total = $this->context->cart->getOrderTotal(true, Cart::BOTH);
        $paytr_transaction->total_paid = 0;
        $paytr_transaction->is_complete = 0;
        $paytr_transaction->is_order = 0;
        $paytr_transaction->is_notify_order = 0;
        $paytr_transaction->is_refunded = 0;
        $paytr_transaction->refund_amount = 0;

        $this->duplicateCart($paytr_transaction);
        $paytr_transaction->date_added = date('Y-m-d H:i:s');
        $paytr_transaction->date_updated = date('Y-m-d H:i:s');
        $paytr_transaction->addTransaction();
    }

    /**
     * Duplicate the current cart for this transaction
     *
     * @param PaytrTransaction $transaction Transaction object
     */
    private function duplicateCart(PaytrTransaction $transaction): void
    {
        $oldCart = new Cart((int)$this->context->cart->id);
        $duplication = $oldCart->duplicate();
        if (!$duplication or !Validate::isLoadedObject($duplication['cart'])) {
            $this->errors[] = Tools::displayError('Sorry, we cannot renew your order.');
        } elseif (!$duplication['success']) {
            $this->errors[] = Tools::displayError('Missing items - we are unable to renew your order');
        } else {
            $this->context->cookie->id_cart = $duplication['cart']->id;
            $this->context->cart = $duplication['cart'];
            $this->context->cookie->write();
            $transaction->id_dup_cart = $duplication['cart']->id;
        }
    }

    /**
     * Assign variables to Smarty
     *
     * @param string|null $errors Error message if any
     * @param string|null $iframe_token IFRAME token if successful
     */
    private function assignSmartyVariables(?string $errors, ?string $iframe_token): void
    {
        $this->context->smarty->assign('ps', [
            'nbProducts' => $this->context->cart->nbProducts(),
            'cust_currency' => $this->context->cart->id_currency,
            'currencies' => $this->module->getCurrency((int)$this->context->cart->id_currency),
            'errors' => $errors,
            'token' => $iframe_token,
            'this_path' => $this->module->getPathUri(),
            'this_path_bw' => $this->module->getPathUri(),
            'this_path_ssl' => Tools::getShopDomainSsl(true, true) . __PS_BASE_URI__ . 'modules/' . $this->module->name . '/',
        ]);
    }

    /**
     * Set the template based on PrestaShop version
     */
    private function setTemplateByVersion(): void
    {
        $this->setTemplate(version_compare(_PS_VERSION_, '1.7.0.0', '>=')
            ? 'module:paytrcheckout/views/templates/front/payment_iframe_plus.tpl'
            : 'payment_iframe.tpl');
    }

    /**
     * Search for category installment setting
     *
     * @param int $category_id Category ID
     * @return int
     */
    public function catSearch($category_id = 0): int
    {
        if (isset($this->category_full[$category_id]) && isset($this->category_installment[$this->category_full[$category_id]])) {
            return $this->category_installment[$this->category_full[$category_id]];
        }

        foreach ($this->category_full as $id => $parent) {
            if ($category_id == $id) {
                if ($parent == 0) {
                    return 0;
                }
                return $this->category_installment[$parent] ?? $this->catSearch($parent);
            }
        }
        return 0;
    }

    /**
     * Get user's IP address
     *
     * @return string
     */
    public function getIp(): string
    {
        return $_SERVER["HTTP_CLIENT_IP"] ?? $_SERVER["HTTP_X_FORWARDED_FOR"] ?? $_SERVER["REMOTE_ADDR"];
    }
}