<?php
/**
 * 2007-2020 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License 3.0 (AFL-3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/AFL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * @author    PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş. <devops@paytr.com>
 * @copyright PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 * @license   https://opensource.org/licenses/AFL-3.0 Academic Free License 3.0 (AFL-3.0)
 * International Registered Trademark & Property of PayTR Ödeme ve Elektronik Para Kuruluşu A.Ş.
 */

require_once(dirname(__FILE__) . '/../../classes/PaytrTransaction.php');

class PaytrCheckoutRefundModuleFrontController extends ModuleFrontController
{
    public function initContent()
    {
        $this->ajax = true;

        parent::initContent();
    }

    /**
     * Display transactions for refund process
     *
     * @throws PrestaShopException
     */
    public function displayAjaxTransactions()
    {
        if (_PS_MODE_DEMO_) {
            throw new PrestaShopException($this->trans('This functionality has been disabled.', [], 'Admin.Notifications.Error'));
        }

        if (!Tools::isSubmit('submitPaytrRefundTable')) {
            $this->renderSubmitControl();
            exit;
        }

        $id_order = Tools::getValue('id_order');
        $order = new Order($id_order);
        $transactions = PaytrTransaction::getTransactionsByCart($order->id_cart);

        $labels = [
            'yes' => $this->module->l('Yes', 'refund'),
            'no' => $this->module->l('No', 'refund'),
            'success' => $this->module->l('Success', 'refund'),
            'failed' => $this->module->l('Failed', 'refund'),
            'partial' => $this->module->l('Partial', 'refund'),
            'full' => $this->module->l('Full', 'refund'),
        ];

        $content = !$transactions
            ? '<tr><td colspan="10" style="text-align: center;">' . $this->l('The transaction is not yet complete.') . '</td></tr>'
            : $this->generateTransactionRows($transactions, $labels, $order);

        $this->renderContent(json_encode(['body' => $content]));
    }

    /**
     * Generate HTML rows for each transaction
     *
     * @param array $transactions Transaction data
     * @param array $labels Translation labels
     * @param Order $order The order object
     * @return string HTML content for transaction rows
     */
    private function generateTransactionRows(array $transactions, array $labels, Order $order): string
    {
        $content = '';
        foreach ($transactions as $transaction) {
            $content .= '<tr>'
                . '<td><a href="https://www.paytr.com/magaza/islemler?merchant_oid=' . $transaction['merchant_oid'] . '" target="_blank">' . $transaction['merchant_oid'] . '</a></td>'
                . '<td>' . $this->renderPrice($transaction['total'], $order->id_currency) . '</td>'
                . '<td>' . $this->renderPrice($transaction['total_paid'], $order->id_currency) . '</td>'
                . '<td>' . ($this->getTransactionStatus($transaction['status'], $labels)) . '</td>'
                . '<td>' . $transaction['status_message'] . '</td>'
                . '<td>' . ($transaction['is_refunded'] ? $labels['yes'] : $labels['no']) . '</td>'
                . '<td>' . $this->getRefundStatus($transaction['refund_status'], $labels) . '</td>'
                . '<td>' . $this->renderPrice($transaction['refund_amount'], $order->id_currency) . '</td>'
                . '<td>' . date('d-m-Y H:i', strtotime($transaction['date_added'])) . '</td>'
                . '<td>' . $this->renderRefundButton($transaction, $order->id_currency) . '</td>'
                . '</tr>';
        }
        return $content;
    }

    /**
     * Get the translation for the transaction status
     *
     * @param string $status Transaction status
     * @param array $labels Translation labels
     * @return string Translated status
     */
    private function getTransactionStatus(string $status, array $labels): string
    {
        return match($status) {
            'success' => $labels['success'],
            'failed' => $labels['failed'],
            default => ''
        };
    }

    /**
     * Get the translation for the refund status
     *
     * @param string $refund_status Refund status
     * @param array $labels Translation labels
     * @return string Translated refund status
     */
    private function getRefundStatus(string $refund_status, array $labels): string
    {
        return match($refund_status) {
            'partial' => $labels['partial'],
            'full' => $labels['full'],
            default => ''
        };
    }

    /**
     * Render refund button if conditions are met
     *
     * @param array $transaction Transaction data
     * @param int $currency_id Currency ID
     * @return string HTML for refund button or empty string
     */
    private function renderRefundButton(array $transaction, int $currency_id): string
    {
        if ($transaction['total_paid'] >= 1 && $transaction['refund_status'] != 'full' && $transaction['status_message'] == 'completed') {
            $amount = $transaction['refund_amount'] < $transaction['total']
                ? $transaction['total'] - $transaction['refund_amount']
                : $transaction['total'];
            return '<button type="button" class="btn btn-warning" data-toggle="modal" data-target="#paytrRefundModal" data-amount="' . round($amount, 2) . '">'
                . $this->module->l('Refund', 'refund')
                . '</button>';
        }
        return '';
    }

    /**
     * Process refund API requests
     *
     * @throws PrestaShopException
     */
    public function displayAjaxRefundApi()
    {
        if (_PS_MODE_DEMO_) {
            throw new PrestaShopException($this->trans('This functionality has been disabled.', [], 'Admin.Notifications.Error'));
        }

        if (!Tools::isSubmit('submitPaytrRefund')) {
            $this->renderSubmitControl();
            exit;
        }

        $amount = Tools::getValue('amount');
        $id_order = Tools::getValue('id_order');

        if ($this->validateRefund($id_order, $amount, $order, $paytr_transaction) === false) {
            exit;
        }

        $merchant = [
            'id' => Configuration::get('PAYTR_MERCHANT_ID'),
            'key' => Configuration::get('PAYTR_MERCHANT_KEY'),
            'salt' => Configuration::get('PAYTR_MERCHANT_SALT'),
        ];

        $paytr_token = base64_encode(hash_hmac('sha256', $merchant['id'] . $paytr_transaction['merchant_oid'] . $amount . $merchant['salt'], $merchant['key'], true));

        $post_val = [
            'merchant_id' => $merchant['id'],
            'merchant_oid' => $paytr_transaction['merchant_oid'],
            'return_amount' => $amount,
            'paytr_token' => $paytr_token,
        ];

        $ch = curl_init();
        curl_setopt_array($ch, [
            CURLOPT_URL => "https://www.paytr.com/odeme/iade",
            CURLOPT_RETURNTRANSFER => 1,
            CURLOPT_POST => 1,
            CURLOPT_POSTFIELDS => $post_val,
            CURLOPT_FRESH_CONNECT => true,
            CURLOPT_TIMEOUT => 90,
            CURLOPT_CONNECTTIMEOUT => 90,
        ]);
        $result = @curl_exec($ch);

        if (curl_errno($ch)) {
            $response = json_encode(['status' => 'error', 'err_msg' => curl_error($ch)]);
        } else {
            $refund_response = json_decode($result, true);
            $refund_status = 'partial';
            $refund_amount = 0;

            if ($refund_response['status'] == 'success') {
                $refund_amount = $this->calculateRefundAmount($paytr_transaction, $refund_response['return_amount']);
                $refund_status = $refund_amount == $paytr_transaction['total'] ? 'full' : 'partial';

                if (Configuration::get('PAYTR_REFUND')) {
                    $order->setCurrentState(7);
                }
                PaytrTransaction::updateTransactionIsRefund($paytr_transaction['merchant_oid'], $refund_amount, $refund_status);
            }

            $refund_response['refund_status'] = $refund_status;
            $response = json_encode($refund_response);
        }
        curl_close($ch);

        $this->renderContent($response);
    }

    /**
     * Validate refund parameters
     *
     * @param string $id_order Order ID
     * @param float $amount Refund amount
     * @param Order &$order Reference to order object
     * @param array &$paytr_transaction Reference to PayTR transaction data
     * @return bool
     */
    private function validateRefund(string $id_order, float $amount, &$order, &$paytr_transaction): bool
    {
        $errors = [
            'id_order' => [
                'empty' => $this->l('Order ID is null!'),
                'invalid' => $this->l('Invalid Order ID!'),
                'not_found' => $this->l('Order not found.'),
            ],
            'amount' => [
                'empty' => $this->l('Amount is null!'),
                'invalid' => $this->l('Invalid Amount!'),
                'zero_or_negative' => $this->l('Amount must be bigger than 0!'),
                'exceeds_available' => $this->l('The amount entered cannot be greater than the remaining amount!'),
            ]
        ];

        if (Tools::isEmpty($id_order)) {
            $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $errors['id_order']['empty']]));
            return false;
        }

        if (!Validate::isInt($id_order)) {
            $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $errors['id_order']['invalid']]));
            return false;
        }

        $order = new Order($id_order);
        if (!Validate::isLoadedObject($order)) {
            $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $errors['id_order']['not_found']]));
            return false;
        }

        if (Tools::isEmpty($amount)) {
            $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $errors['amount']['empty']]));
            return false;
        }

        $amount = (float)str_replace(['+', ','], ['', '.'], $amount);

        if (!Validate::isPrice($amount)) {
            $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $errors['amount']['invalid']]));
            return false;
        }

        if ($amount <= 0) {
            $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $errors['amount']['zero_or_negative']]));
            return false;
        }

        $paytr_transaction = PaytrTransaction::getTransactionByCart($order->id_cart);
        $available_amount = $paytr_transaction['is_refunded'] && $paytr_transaction['refund_status'] == 'partial'
            ? $paytr_transaction['total'] - $paytr_transaction['refund_amount']
            : $paytr_transaction['total'];

        if ($amount > round($available_amount, 2)) {
            $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $errors['amount']['exceeds_available']]));
            return false;
        }

        return true;
    }

    /**
     * Calculate the total refund amount after the current refund
     *
     * @param array $transaction Transaction data
     * @param float $refunded_amount Amount to be refunded
     * @return float Total refunded amount
     */
    private function calculateRefundAmount(array $transaction, float $refunded_amount): float
    {
        if ($transaction['total'] == $refunded_amount) {
            return $refunded_amount;
        }

        $total_refunded = $transaction['is_refunded'] && $transaction['refund_status'] == 'partial'
            ? $transaction['refund_amount'] + $refunded_amount
            : $refunded_amount;

        return round($total_refunded, 2);
    }

    /**
     * Render the JSON response and terminate execution
     *
     * @param string $response JSON response to send
     */
    protected function renderContent(string $response): void
    {
        die($response);
    }

    /**
     * Format price based on PrestaShop version
     *
     * @param float $price Price to format
     * @param int $currency_id ID of the currency
     * @return float|string Formatted price
     */
    protected function renderPrice(float $price, int $currency_id): float|string
    {
        $currency = new Currency($currency_id);
        return version_compare(_PS_VERSION_, '1.7.6.0', '>=')
            ? Context::getContext()->currentLocale->formatPrice($price, $currency->iso_code)
            : Tools::displayPrice($price, $currency);
    }

    /**
     * Render error message for submit control
     */
    protected function renderSubmitControl(): void
    {
        $this->renderContent(json_encode(['status' => 'error', 'err_msg' => $this->l('Service is not found!')]));
    }
}